{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ Purpose: Declaration of data type for representation of a complex interval }
{    polynomial by its coefficients, and of I/O procedures for this data     }
{    type. Boolean operator in and function for epsilon inflation are        }
{    supplied                                                                }
{ Global types, operators and procedures:                                    }
{    type CIPolynomial    : representation of complex interval polynomials   }
{    operator :=          : assignment of complex interval polynomials by a  }
{                           real value                                       }
{    procedure read(...)  : input of data type CIPolynomial                  }
{    procedure write(...) : output of data type CIPolynomial                 }
{    operator in          : boolean-valued operator determining whether a    }
{                           complex interval polynomial lies (componentwise) }
{                           in another complex interval polynomial           }
{    function blow        : componentwise epsilon inflation of a complex     }
{                           interval polynomial                              }
{ Remark: Variables of type 'CIPolynomial' should be declared with lower     }
{    bound 0 (zero).                                                         }
{----------------------------------------------------------------------------}
module cipoly;

use
  iostd,   { Needed for abnormal termination with 'exit' }
  i_ari,   { Interval arithmetic         }
  ci_ari;  { Complex interval arithmetic }

global type
  CIPolynomial  = global dynamic array[*] of cinterval;

global operator := ( var p : CIPolynomial; r : real );
var
  i : integer;
begin
  for i := lb(p) to ub(p) do p[i] := r;
end;

global procedure read ( var t : text; var p : CIPolynomial );
var
  i : integer;
begin
  if (lb(p) <> 0) then
  begin
    write('Error: Variable of type CIPolynomial was declared with ');
    writeln('lower bound <> 0!');
    exit(-1); { Abnormal program termination }
  end;
  write('  x^0 * '); read(t,p[0]);
  for i := 1 to ub(p) do
    begin  write('+ x^',i:0,' * '); read(t,p[i])  end;
end;

global procedure write ( var t : text; p : CIPolynomial );
var
  i          : integer;
  PolyIsZero : boolean;   { Signals 'p' is a zero polynomial }
begin
  PolyIsZero := true;
  for i := 0 to ub(p) do
    if (p[i] <> 0) then
    begin
      if PolyIsZero then write(t,'  ')  else  write(t,'+ ');
      writeln(t,'(',  re(p[i]),',');
      writeln(t,'   ',im(p[i]),' ) * x^',i:1);
      PolyIsZero := false;
    end;
  if PolyIsZero then writeln(t,'   0 (= zero polynomial)');
end;

global operator in ( p, q : CIPolynomial ) res : boolean;
var
  i    : integer;
  incl : boolean;
begin
  incl := true;  i := 0;
  while (incl = true) and (i <= ub(p)) do
    begin  incl := p[i] in q[i];  i := i+1   end;
  res  :=  incl;
end;

global function blow ( p   : CIPolynomial;
                       eps : real           ) : CIPolynomial[0..ub(p)];
var
  i : integer;
begin
  for i := 0 to ub(p) do blow[i] := blow(p[i],eps);
end;


{----------------------------------------------------------------------------}
{ Module initialization part                                                 }
{----------------------------------------------------------------------------}
begin
  { Nothing to initialize }
end.
