{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ Purpose: Computation of an enclosure of the value of a real arithmetic     }
{    expression composed of the operations +, -, *, /, and ^, where ^        }
{    denotes exponentiation by an integer.                                   }
{ Method: Iterative refinement using a defect correction mechanism. The      }
{    successively computed corrections for the result are stored in a        }
{    staggered correction format.                                            }
{ Global types, operators, functions, and procedures:                        }
{   type      Staggered  :  Data type for staggered data representation      }
{   type      StaggArray :  Array of type 'Staggered'                        }
{   operators :=         :  Assignment of 'Staggered' := 'real'              }
{   operators +, -, *, / :  Both operands of type 'Staggered' or one of      }
{                           type 'Staggered' and one of type 'real'          }
{   function  Power(...) :  Argument of type 'Staggered', exponent of type   }
{                           'integer' (Exponentiation by an integer)         }
{   procedure Eval(...)  :  Main procedure for evaluation of an expression   }
{----------------------------------------------------------------------------}
module expreval;   { Expression evaluation }
                   { ----       ----       }
use
  x_real,   { Needed if a signaling NaN is to be returned }
  i_util,   { Utilities of type interval                  }
  i_ari;    { Interval arithmetic                         }

const
  MaxStaggPrec = 10;    { Maximum number of staggered corrections which can  }
                        { be stored in a variable of type 'Staggered'.       }
                        { Recompile this module if its value is changed.     }
global type
  Staggered  = record
                 Val : array[0..MaxStaggPrec] of real;  { Stagg. corrections }
                 Err : interval;                        { Error enclosure    }
               end;
  StaggArray = global dynamic array [*] of Staggered;

type
  IntResPtr = ^IntRes;   { Pointer to an intermediate result }
  IntRes    = record
                Entry : Staggered;         { Staggered entry }
                Next  : IntResPtr;         { Next entry      }
              end;

{----------------------------------------------------------------------------}
{ Private variables which are globally used within other functions and       }
{ procedures of this module.                                                 }
{----------------------------------------------------------------------------}
var
  DivByZero    : boolean;     { Error flag                                }
  InitFlag     : boolean;     { Signals the initialization process        }
  ActStaggPrec : integer;     { Actual length of the staggered format     }
  HeadPtr      : IntResPtr;   { Head of the list of intermediate results  }
  ActPtr       : IntResPtr;   { Pointer to the actual intermediate result }
  FreePtr      : IntResPtr;   { Pointer to freed intermediate results     }

{----------------------------------------------------------------------------}
{ Error codes used in this module.                                           }
{----------------------------------------------------------------------------}
const
  NoError   = 0;   { No error occurred.                                }
  ItFailed  = 1;   { Maximum number of staggered corrections exceeded. }
  ZeroDiv   = 2;   { Division by zero that could not be                }
                   { removed by iterative refinement.                  }

{----------------------------------------------------------------------------}
{ Error messages depending on the error code.                                }
{----------------------------------------------------------------------------}
global function EvalErrMsg ( ErrNo : integer ) : string;
var
  Msg : string;
begin
  case ErrNo of
    NoError  : Msg := '';
    ItFailed : Msg := 'Maximum number of staggered corrections (=' +
                      image(MaxStaggPrec) + ') exceeded';
    ZeroDiv  : Msg := 'Division by zero occurred';
    else     : Msg := 'Code not defined';
  end;
  if (ErrNo <> NoError) then Msg := 'Error: ' + Msg + '!';
  EvalErrMsg := Msg;
end;

{----------------------------------------------------------------------------}
{ Since the intermediate results of operations for staggered data must be    }
{ iteratively updated, it is necessary to store these data. For this         }
{ purpose, a linear linked list is used. Its head is accessed via 'HeadPtr', }
{ whereas its actual entry is accessed via 'ActPtr'. 'FreePtr' is a pointer  }
{ to a list of already allocated but actually unused entries. To prevent     }
{ creation of garbage in memory, the entries of this list are used first     }
{ when allocating a new intermediate result. All procedures for the handling }
{ of the list of intermediate results are locally defined.                   }
{----------------------------------------------------------------------------}
procedure InitList;                { Initialize list of intermediate results }
begin                              {-----------------------------------------}
  DivByZero := false;
  if (HeadPtr <> nil) then     { Use list of freed entries }
    FreePtr := HeadPtr
  else                         { A list was not yet allocated }
    FreePtr := nil;
  HeadPtr := nil;
  ActPtr := nil;
end;

procedure ResetList;                   { Reset error flag and set the actual }
begin                                  { pointer to the head of the list.    }
  DivByZero := false;                  {-------------------------------------}
  ActPtr := HeadPtr;
end;

procedure AllocEntry ( var p : IntResPtr );    { Allocate memory for a new   }
begin                                          { entry. Use a previously     }
  if (FreePtr <> nil) then                     { created entry from the list }
    begin                                      { of freed entries if any.    }
      p := FreePtr;                            {-----------------------------}
      FreePtr := FreePtr^.next;
    end
  else
    new(p);
end;

procedure InitEntry ( Approx : real );   { Get a new list entry and initial- }
var                                      { ize its first staggered component }
  p : IntResPtr;                         { with the value 'Approx'.          }
begin                                    {-----------------------------------}
  AllocEntry(p);
  p^.Entry.Val[0] := Approx;
  p^.Next := nil;
  if (HeadPtr = nil) then
    begin  HeadPtr := p;  ActPtr := p;  end
  else
    begin  ActPtr^.Next := p;  ActPtr := p;  end;
end;

procedure UpdateError ( Error : interval );     { Update the error component }
begin                                           { of the actual entry.       }
  ActPtr^.Entry.Err := Error;                   {----------------------------}
  ActPtr := ActPtr^.Next;
end;

procedure UpdateStaggComp ( i : integer );      { Update the i-th staggered  }
begin                                           { component of all list      }
  ActPtr := HeadPtr;                            { entries by the midpoint of }
  while (ActPtr <> nil) do                      { the error interval.        }
  begin                                         {----------------------------}
    with ActPtr^.Entry do
    begin
      Val[i] := mid(Err);  Err := 0;
    end;
    ActPtr := ActPtr^.Next;
  end;
end;

{----------------------------------------------------------------------------}
{ Assignment operator used to convert a real input parameter 'r' to the      }
{ staggered type. A real operand is assumed to be exact! It must not have    }
{ been rounded. In particular, 'r' cannot be                                 }
{    - a real value such as 0.1 which is not exactly representable in the    }
{      internal binary format,                                               }
{    - rounded by conversion to the internal binary data format during       }
{      input,                                                                }
{    - any arithmetic expression which cannot be evaluated exactly.          }
{----------------------------------------------------------------------------}
global operator := ( var x : Staggered; r : real );
var
  i : integer;
begin
  x.Val[0] := r;
  for i := 1 to MaxStaggPrec do x.Val[i] := 0;
  x.Err := 0;
end;

{----------------------------------------------------------------------------}
{ Arithmetic operators +, -, *, and / for operands of type 'Staggered'. A    }
{ division by an interval containing zero will be avoided and is marked by   }
{ setting the 'DivByZero' flag. If 'DivByZero' is set, all succeeding opera- }
{ tions are not executed. The evaluation is stopped at this point but may be }
{ restarted after updating a new staggered component by the midpoints of the }
{ error enclosures computed so far.                                          }
{----------------------------------------------------------------------------}
global operator + ( x, y : Staggered ) AddStaggStagg : Staggered;
var
  i, p : integer;
  z    : Staggered;
begin
  if (not DivByZero) then
  begin
    if (InitFlag) then   { Initialize first staggered component }
      begin
        z.Val[0] := x.Val[0] + y.Val[0];
        InitEntry(z.Val[0]);
      end
    else
      begin                                     
        z := ActPtr^.Entry;  p := ActStaggPrec;  { Get actual values of z }

        { Error:  dz := ##( x + y - z + dx + dy ) }
        z.Err := ##( for i:=0 to p sum( x.Val[i] + y.Val[i] - z.Val[i] ) +
                     x.Err + y.Err );
        UpdateError(z.Err);
      end;
    AddStaggStagg := z;
  end;
end; {operator +}

global operator - ( x, y : Staggered ) SubStaggStagg : Staggered;
var
  i, p : integer;
  z    : Staggered;
begin
  if (not DivByZero) then
  begin
    if (InitFlag) then   { Initialize first staggered component }
      begin
        z.Val[0] := x.Val[0] - y.Val[0];
        InitEntry(z.Val[0]);
      end
    else
      begin
        z := ActPtr^.Entry;  p := ActStaggPrec;  { Get actual values of z }

        { Error:  dz := ##( x - y - z + dx - dy ) }
        z.Err := ##( for i:=0 to p sum( x.Val[i] - y.Val[i] - z.Val[i] ) +
                     x.Err - y.Err );
        UpdateError(z.Err);
      end;
    SubStaggStagg := z;
  end;
end; {operator -}

global operator * ( x, y : Staggered ) MulStaggStagg : Staggered;
var
  i, j, p : integer;
  z       : Staggered;
begin
  if (not DivByZero) then
  begin
    if (InitFlag) then   { Initialize first staggered component }
      begin
        z.Val[0] := x.Val[0] * y.Val[0];
        InitEntry(z.Val[0]);
      end
    else
      begin
        z := ActPtr^.Entry;  p := ActStaggPrec;  { Get actual values of z }

        { Error:  dz := ##( x*y - z + y*dx + x*dy + dx*dy ) }
        z.Err := ##( for i:=0 to p sum( for j:=0 to p sum(
                       x.Val[i]*y.Val[j] ) - z.Val[i] ) +
                     for i:=0 to p sum(
                       y.Val[i]*x.Err + x.Val[i]*y.Err ) +
                     x.Err*y.Err );
        UpdateError(z.Err);
      end;
    MulStaggStagg := z;
  end;
end; {operator *}

global operator / ( x, y : Staggered ) DivStaggStagg : Staggered;
var
  i, j, p    : integer;
  z          : Staggered;
  num, denom : interval;
begin
  if (not DivByZero) then
  begin
    if (InitFlag) then   { Initialize first staggered component }
      if (y.Val[0] <> 0) then
        begin
          z.Val[0] := x.Val[0] / y.Val[0];
          InitEntry(z.Val[0]);
        end
      else
        begin
          DivByZero := true;
          InitEntry(0);
        end
    else
      begin
        z := ActPtr^.Entry;  p := ActStaggPrec;  { Get actual values of z }

        { Error:  dz := ##( x - z*y + dx - z*dy )  /  ##( y + dy ) }
        num := ##( for i:=0 to p sum( x.Val[i] - for j:=0 to p sum(
                     y.Val[j]*z.Val[i] ) ) +
                   x.Err - for i:=0 to p sum( z.Val[i]*y.Err ) );
        denom := ##( for i:=0 to p sum( y.Val[i] ) + y.Err );

        if (0 in denom) then
          begin  z.Err := 0;  DivByZero:= true;  end
        else
          z.Err := num / denom;

        UpdateError(z.Err);
      end;
    DivStaggStagg := z;
  end;
end; {operator /}

{----------------------------------------------------------------------------}
{ Arithmetic operators for different operands the one of type 'real' and the }
{ other one of type 'Staggered'. All these operators are implemented by      }
{ first coercing both operands to 'Staggered' type and then calling the      }
{ corresponding operators for the type 'Staggered'.                          }
{----------------------------------------------------------------------------}
global operator + ( x : real; y : Staggered ) AddRealStagg : Staggered;
var
  z : Staggered;
begin
  z := x;  AddRealStagg := z + y;
end;

global operator + ( x : Staggered; y : real ) AddStaggReal : Staggered;
var
  z : Staggered;
begin
  z := y;  AddStaggReal := x + z;
end;

global operator - ( x : real; y : Staggered ) SubRealStagg : Staggered;
var
  z : Staggered;
begin
  z := x;  SubRealStagg := z - y;
end;

global operator - ( x : Staggered; y : real ) SubStaggReal : Staggered;
var
  z : Staggered;
begin
  z := y;  SubStaggReal := x - z;
end;

global operator * ( x : real; y : Staggered ) MulRealStagg : Staggered;
var
  z : Staggered;
begin
  z := x;  MulRealStagg := z * y;
end;

global operator * ( x : Staggered; y : real ) MulStaggReal : Staggered;
var
  z : Staggered;
begin
  z := y;  MulStaggReal := x * z;
end;

global operator / ( x : real; y : Staggered ) DivRealStagg : Staggered;
var
  z : Staggered;
begin
  z := x;  DivRealStagg := z / y;
end;

global operator / ( x : Staggered; y : real ) DivStaggReal : Staggered;
var
  z : Staggered;
begin
  z := y;  DivStaggReal := x / z;
end;

{----------------------------------------------------------------------------}
{ Power function for integer exponents using the binary shift method. Thus,  }
{ the number of successive multiplications is reduced from n to log(2,n).    }
{ Since x^n is considered to be a monomial, we define x^0 := 1.              }
{----------------------------------------------------------------------------}
global function Power ( x : Staggered; n : integer ) : Staggered;
var
  m    : integer;
  p, z : Staggered;
begin
  if (not DivByZero) then
  begin
    if (n = 0) then
      Power := 1
    else
      begin
        if (n > 0) then m := n  else  m := -n;

        p := 1;  z := x;                       { Binary shift method }
        while (m > 0) do                       {---------------------}
        begin
          if (m mod 2 = 1) then p := p * z;
          m := m div 2;
          if (m > 0) then z := z * z;
        end;

        if (n > 0) then  Power := p  else  Power := 1/p;
      end;
  end;
end; {function Power}

{----------------------------------------------------------------------------}
{ Purpose: The procedure 'Eval' may be used for the computation of an        }
{    enclosure of the value of a real arithmetic expression composed of the  }
{    operations +, -, *, /, and ^, where ^ denotes exponentiation by an      }
{    integer.                                                                }
{ Parameters:                                                                }
{    In  : 'f'         : a function of type 'Staggered' whose arguments are  }
{                        passed in an array of type 'Staggered'.             }
{          'Arg'       : the real-valued arguments of 'f' stored as          }
{                        components of a real vector.                        }
{          'Eps'       : desired accuracy.                                   }
{    Out : 'Approx'    : result computed with standard floating-point        }
{                        arithmetic.                                         }
{          'Encl'      : verified enclosure of the result.                   }
{          'StaggPrec' : number of corrections needed.                       }
{          'Err'       : error code.                                         }
{ Description:                                                               }
{   The expression 'f' is evaluated for the real arguments which are stored  }
{   sequentially in the vector 'Arg'. Initially, the real arguments are      }
{   converted to arguments of type 'Staggered'. When 'f' is evaluated for    }
{   the first time using the special staggered arithmetic, the list of       }
{   intermediate results is initialized. Each time 'f' is evaluated again,   }
{   the error of every intermediate result is enclosed. The midpoints of     }
{   these enclosures are used to update the intermediate results. The        }
{   iteration is finished if the error of the last intermediate result       }
{   (= value of 'f') is less than the desired accuracy. Otherwise, the       }
{   iteration is halted after 'MaxStaggPrec' steps.                          }
{----------------------------------------------------------------------------}
global procedure Eval ( function f ( v : StaggArray ) : Staggered;
                            Arg       : rvector;
                            Eps       : real;
                        var Approx    : real;
                        var Encl      : interval;
                        var StaggPrec : integer;
                        var Err       : integer);
var
  i        : integer;
  x        : StaggArray[1..ub(Arg)-lb(Arg)+1];
  StaggRes : Staggered;
  Success  : boolean;
begin
  for i := 1 to ub(Arg)-lb(Arg)+1 do                  { Initialize arguments }
    x[i] := Arg[lb(Arg)+i-1];

  InitList;                     { Initialize list for intermediate results.  }
  InitFlag := true;             { So far, no staggered corrections computed. }
  StaggRes := f(x);             { Compute first staggered component.         }

  { In general, the first component of 'StaggRes' will now hold the usual    }
  { floating-point approximation, except when a division by zero occurred.   }
  { In the latter case, a signaling NaN (= Not a Number, for more details    }
  { see the PASCAL-XSC user's guide) is returned.                            }
  {--------------------------------------------------------------------------}
  if (DivByZero) then
    Approx := x_value(x_sNaN)
  else
    Approx := StaggRes.Val[0];
                                                { Initial approximations are }
  InitFlag := false;  ActStaggPrec := 0;        { already computed.          }
  repeat
    ResetList;                { Compute new enclosures of the absolute error }
    StaggRes := f(x);         { of any intermediate result.                  }

    Encl := ##(for i:=0 to ActStaggPrec sum(StaggRes.Val[i]) + StaggRes.Err);

    Success := (not DivByZero and (RelDiam(Encl) <= Eps) );

    { Increment actual staggered precision and store the next component of   }
    { the intermediate results by updating the list of intermediate results. }
    {------------------------------------------------------------------------}
    if ( (not Success) and (ActStaggPrec < MaxStaggPrec) ) then
    begin
      ActStaggPrec := ActStaggPrec + 1;
      UpdateStaggComp(ActStaggPrec);
    end;
  until ( Success or (ActStaggPrec = MaxStaggPrec) );
  StaggPrec := ActStaggPrec;

  if (Success) then        { Set error code }
    Err := NoError
  else if (DivByZero) then
    Err := ZeroDiv
  else
    Err := ItFailed;
end; {procedure Eval}

{----------------------------------------------------------------------------}
{ Module initialization part                                                 }
{----------------------------------------------------------------------------}
begin
  { Nothing to initialize }
end.
