(* RCS $Id: x_real.p,v 1.1.1.1 2003/11/20 08:49:04 hoeffgen Exp $ *)
(*-----------------------------------------------------------*)
(* DESCRIPTION : module for special routines for real values *)
(*-----------------------------------------------------------*)
{---------------------------------------------------------------
 |      x_real : module for special routines for real values   |
 |      ======                                                 |
 |                                                             |
 |      Constants:    - IEEE_DIV_BY_ZERO  (code)               |
 |                      IEEE_INEXACT      (code)               |
 |                      IEEE_INV_OP       (code)               |
 |                      IEEE_OVERFLOW     (code)               |
 |                      IEEE_UNDERFLOW    (code)               |
 |                    - IEEE_CONTINUE     (action)             |
 |                    - MAXREAL           (real)               |
 |                      MINREAL           (real)               |
 |      Types :       - x_ccode                                |
 |      Functions :   - x_class(r)   ==> x_ccode               |
 |                      x_value(x_ccode) ==> REAL              |
 |                    - x_comp(r,i)  ==> REAL                  |
 |                      x_expo(r)    ==> INTEGER               |
 |                      x_mant(r)    ==> REAL                  |
 |                    - x_sqrt(r)    ==> REAL/INTERVAL         |
 |                      x_exp(r)     ==> REAL/INTERVAL         |
 |                      x_exp2(r)    ==> REAL/INETRVAL         |
 |                      x_exp10(r)   ==> REAL/INTERVAL         |
 |                      x_ln(r)      ==> REAL/INTERVAL         |
 |                      x_log2(r)    ==> REAL/INTERVAL         |
 |                      x_log10(r)   ==> REAL/INTERVAL         |
 |                      x_sin(r)     ==> REAL/INTERVAL         |
 |                      x_cos(r)     ==> REAL/INTERVAL         |
 |                      x_tan(r)     ==> REAL/INTERVAL         |
 |                      x_cot(r)     ==> REAL/INTERVAL         |
 |                      x_arcsin(r)  ==> REAL/INTERVAL         |
 |                      x_arccos(r)  ==> REAL/INTERVAL         |
 |                      x_arctan(r)  ==> REAL/INTERVAL         |
 |                      x_arctan2(r1,r2) ==> REAL/INTERVAL     |
 |                      x_arccot(r)  ==> REAL/INTERVAL         |
 |                      x_sinh(r)    ==> REAL/INTERVAL         |
 |                      x_cosh(r)    ==> REAL/INTERVAL         |
 |                      x_tanh(r)    ==> REAL/INTERVAL         |
 |                      x_coth(r)    ==> REAL/INTERVAL         |
 |                      x_arsinh(r)  ==> REAL/INTERVAL         |
 |                      x_arcosh(r)  ==> REAL/INTERVAL         |
 |                      x_artanh(r)  ==> REAL/INTERVAL         |
 |                      x_arcoth(r)  ==> REAL/INTERVAL         |
 |                    - IEEE_test(code) ==> BOOLEAN            |
 |                    - x_rval ==> hex real string to real     |
 |                      x_image ==> real to hex real string    |
 |      Procedures:   - IEEE_environment(action,code,mode)     |
 |                      IEEE_trap_enable(code,mode)            |
 |                      IEEE_set(code)                         |
 |                      IEEE_reset(code)                       |
 |                      IEEE_save(var x : integer)             |
 |                      IEEE_restore( x : integer)             |
 |                    - read(f,r:'x')                          |
 |                      write(f,r:'x')                         |
 |                                                             |
 ---------------------------------------------------------------}
MODULE x_real;

{---------------------------------------------------------------
 |      Global constants                                       |
 |                                                             |
 |         IEEE_DIV_BY_ZERO = DIVision BY ZERO exception       |
 |         ================                                    |
 |         IEEE_INEXACT     = INEXACT result exception         |
 |         ============                                        |
 |         IEEE_INV_OP      = INValid OPeration exception      |
 |         ===========                                         |
 |         IEEE_OVERFLOW    = exponent OVERFLOW exception      |
 |         =============                                       |
 |         IEEE_UNDERFLOW   = exponent UNDERFLOW exception     |
 |         ==============                                      |
 |         IEEE_ALL         = all IEEE exceptions              |
 |         ========                                            |
 |         MAXREAL          = largest positive real value      |
 |         =======                                             |
 |         MINREAL          = smallest positive real value     |
 |         =======                                             |
 ---------------------------------------------------------------}
GLOBAL CONST 
		IEEE_INV_OP        =  256; { INV_OP }
	     IEEE_DIV_BY_ZERO   = 2560; { DIV_BY_ZERO }
	     IEEE_OVERFLOW      = 2816; { OVERFLOW }
	     IEEE_UNDERFLOW     = 3072; { UNDERFLOW }
	     IEEE_INEXACT       = 3328; { INEXACT }
	     IEEE_CONTINUE      =   64; { E_CONT }
          IEEE_ALL           =    0; { all exceptions }
          MAXREAL            = 1.7976931348623158e308;
          MINREAL            = 4.9406564584124654e-324;

{---------------------------------------------------------------
 |      x_ccode : Classification codes for IEEE real values    |
 |      =======                                                |
 |                                                             |
 |      Note :  Do not change the order of enumeration names!  |
 ---------------------------------------------------------------}
GLOBAL TYPE x_ccode = GLOBAL
                      ( x_sNaN,  { signaling NaN         }
			x_qNaN,  { quiet NaN             }
			x_minf,  { minus infinity        }
			x_mnor,  { negative normalized   }
			x_mden,  { negative denormalized }
			x_mnul,  { minus zero            }
			x_pnul,  { plus zero             }
			x_pden,  { positive denormalized }
			x_pnor,  { positive normalized   }
			x_pinf   { plus infinity         }
		      );

{---------------------------------------------------------------
 |      local definitions/declarations                         |
 ---------------------------------------------------------------}
PROCEDURE e_sioe; EXTERNAL e_sioe;
PROCEDURE e_rioe; EXTERNAL e_rioe;
PROCEDURE e_siee; EXTERNAL e_siee;
PROCEDURE e_riee; EXTERNAL e_riee;
PROCEDURE e_sofe; EXTERNAL e_sofe;
PROCEDURE e_rofe; EXTERNAL e_rofe;
PROCEDURE e_sufe; EXTERNAL e_sufe;
PROCEDURE e_rufe; EXTERNAL e_rufe;
PROCEDURE e_sdze; EXTERNAL e_sdze;
PROCEDURE e_rdze; EXTERNAL e_rdze;
PROCEDURE e_sioo; EXTERNAL e_sioo;
PROCEDURE e_rioo; EXTERNAL e_rioo;
PROCEDURE e_sieo; EXTERNAL e_sieo;
PROCEDURE e_rieo; EXTERNAL e_rieo;
PROCEDURE e_sofo; EXTERNAL e_sofo;
PROCEDURE e_rofo; EXTERNAL e_rofo;
PROCEDURE e_sufo; EXTERNAL e_sufo;
PROCEDURE e_rufo; EXTERNAL e_rufo;
PROCEDURE e_sdzo; EXTERNAL e_sdzo;
PROCEDURE e_rdzo; EXTERNAL e_rdzo;
PROCEDURE e_rall; EXTERNAL e_rall;
PROCEDURE e_sall; EXTERNAL e_sall;
FUNCTION e_io_o : BOOLEAN; EXTERNAL e_io_o;
FUNCTION e_ie_o : BOOLEAN; EXTERNAL e_ie_o;
FUNCTION e_of_o : BOOLEAN; EXTERNAL e_of_o;
FUNCTION e_uf_o : BOOLEAN; EXTERNAL e_uf_o;
FUNCTION e_dz_o : BOOLEAN; EXTERNAL e_dz_o;
PROCEDURE e_ienv(action,code : INTEGER; mode : BOOLEAN); EXTERNAL e_ienv;

{---------------------------------------------------------------
 |      x_class : function returns classification code         |
 |      =======   for a real value                             |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_class ( r : REAL ) : x_ccode;
   EXTERNAL r_clss;

{---------------------------------------------------------------
 |      x_value : function returns a real value related to the |
 |      =======   specified classification code.               |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_value ( r : x_ccode ) : REAL;
   EXTERNAL r_valu;

{---------------------------------------------------------------
 |      x_comp : Compose a real value from mantissa m and      |
 |      ======   exponent e                                    |
 |                                                             |
 |      Note : The magnitude of a non-zero mantissa m is       |
 |             assumed to be greater than or equal to 1.0      |
 |             and less than 2.0!                              |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_comp ( m : REAL; e : INTEGER ) : REAL;
BEGIN
   IF (e>-MAXINT) AND (e<MAXINT) THEN e := e+1;
   x_comp := COMP(m*0.5,e);
END;

{---------------------------------------------------------------
 |      x_expo : Function returns the exponent of a real value |
 |      ======                                                 |
 |                                                             |
 |      Note : The magnitude of a non-zero mantissa m is       |
 |             assumed to be greater than or equal to 1.0      |
 |             and less than 2.0!                              |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_expo ( r : REAL ) : INTEGER;
   VAR i : INTEGER;
BEGIN
   i := EXPO(r);
   IF (i=-MAXINT) OR (i=MAXINT) THEN
      x_expo := i
   ELSE
      x_expo := i-1;
END;

{---------------------------------------------------------------
 |      x_mant : Function returns the mantissa of a real value |
 |      ======                                                 |
 |                                                             |
 |      Note : The magnitude of a non-zero mantissa m is       |
 |             assumed to be greater than or equal to 1.0      |
 |             and less than 2.0!                              |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_mant ( r : REAL ) : REAL;
   VAR m : REAL;
BEGIN
   m := MANT(r)*2.0;
   x_mant := m;
END;

{---------------------------------------------------------------
 |      read : read a real value using representation          |
 |      ====   according to 'mode'.                            |
 |             mode = 'x','X' : 16-digit hexadecimal notation  |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE read( VAR f : TEXT; VAR r : REAL; mode : CHAR );
   EXTERNAL f_rhex;

{---------------------------------------------------------------
 |      write : write a real value using representation        |
 |      =====   according to 'mode'.                           |
 |              mode = 'x' : 16-digit hexadecimal notation     |
 |                           with lower case letters           |
 |              mode = 'X' : 16-digit hexadecimal notation     |
 |                           with upper case letters           |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE write( VAR f : TEXT; r : REAL; mode : CHAR );
   EXTERNAL f_whex;

{---------------------------------------------------------------
 |      IEEE_test : Test IEEE flag for exception identified    |
 |      =========   by 'code'. Function value is TRUE if flag  |
 |                  is set, otherwise FALSE.                   |
 |      code   : one of                                        |
 |                IEEE_DIV_BY_ZERO = DIVision BY ZERO          |
 |                IEEE_INEXACT     = INEXACT result            |
 |                IEEE_INV_OP      = INValid OPeration         |
 |                IEEE_OVERFLOW    = exponent OVERFLOW         |
 |                IEEE_UNDERFLOW   = exponent UNDERFLOW        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION IEEE_test( code : integer ) : BOOLEAN;
BEGIN
   CASE code OF
   IEEE_DIV_BY_ZERO : IEEE_test := e_dz_o;
   IEEE_INEXACT     : IEEE_test := e_ie_o;
   IEEE_INV_OP      : IEEE_test := e_io_o;
   IEEE_OVERFLOW    : IEEE_test := e_of_o;
   IEEE_UNDERFLOW   : IEEE_test := e_uf_o;
   ELSE             : IEEE_test := FALSE;
   END;
END;

{---------------------------------------------------------------
 |      IEEE_set : Set IEEE flag for exception identified      |
 |      ========   by 'code'.                                  |
 |      code   : one of                                        |
 |                IEEE_DIV_BY_ZERO = DIVision BY ZERO          |
 |                IEEE_INEXACT     = INEXACT result            |
 |                IEEE_INV_OP      = INValid OPeration         |
 |                IEEE_OVERFLOW    = exponent OVERFLOW         |
 |                IEEE_UNDERFLOW   = exponent UNDERFLOW        |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_set( code : integer );
BEGIN
   CASE code OF
   IEEE_DIV_BY_ZERO : e_sdzo;
   IEEE_INEXACT     : e_sieo;
   IEEE_INV_OP      : e_sioo;
   IEEE_OVERFLOW    : e_sofo;
   IEEE_UNDERFLOW   : e_sufo;
   IEEE_ALL         : e_sall;
   ELSE             : ;
   END;
END;

{---------------------------------------------------------------
 |      IEEE_reset : Reset IEEE flag for exception identified  |
 |      ==========   by 'code'.                                |
 |      code   : one of                                        |
 |                IEEE_DIV_BY_ZERO = DIVision BY ZERO          |
 |                IEEE_INEXACT     = INEXACT result            |
 |                IEEE_INV_OP      = INValid OPeration         |
 |                IEEE_OVERFLOW    = exponent OVERFLOW         |
 |                IEEE_UNDERFLOW   = exponent UNDERFLOW        |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_reset( code : integer );
BEGIN
   CASE code OF
   IEEE_DIV_BY_ZERO : e_rdzo;
   IEEE_INEXACT     : e_rieo;
   IEEE_INV_OP      : e_rioo;
   IEEE_OVERFLOW    : e_rofo;
   IEEE_UNDERFLOW   : e_rufo;
   IEEE_ALL         : e_rall;
   ELSE             : ;
   END;
END;

{---------------------------------------------------------------
 |      IEEE_restore : Restore IEEE exception flags and trap   |
 |      ============   enabled flags from value x.             |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_restore( x : integer );
   EXTERNAL e_rest;

{---------------------------------------------------------------
 |      IEEE_save : Save IEEE exception flags and trap enabled |
 |      =========   flags in variable x.                       |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_save( var x : integer );
   EXTERNAL e_save;

{---------------------------------------------------------------
 |      IEEE_environment : Transfer of an 'action' code to     |
 |      ================   certain IEEE exception environments |
 |                         represented by 'code'. The final    |
 |                         status is specified by 'mode'.      |
 |      action : IEEE_CONTINUE                                 |
 |                Continue processing for exceptions in 'code' |
 |                if 'mode' is TRUE.                           |
 |                Abort processing for exceptions in 'code'    |
 |                if 'mode' is FALSE.                          |
 |      code   : one of                                        |
 |                IEEE_DIV_BY_ZERO = DIVision BY ZERO          |
 |                IEEE_INEXACT     = INEXACT result            |
 |                IEEE_INV_OP      = INValid OPeration         |
 |                IEEE_OVERFLOW    = exponent OVERFLOW         |
 |                IEEE_UNDERFLOW   = exponent UNDERFLOW        |
 |      mode   : TRUE  = status becomes activated              |
 |               FALSE = status becomes deactivated            |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_environment(action,code : INTEGER; mode : BOOLEAN);
BEGIN
   IF (action=IEEE_CONTINUE) AND
      ((code=IEEE_INV_OP)      OR
       (code=IEEE_DIV_BY_ZERO) OR
       (code=IEEE_OVERFLOW)    OR
       (code=IEEE_UNDERFLOW)   OR
       (code=IEEE_INEXACT))
   THEN
      e_ienv(action,code,mode)
END;

{---------------------------------------------------------------
 |      IEEE_trap_enable : Toggle the enabled status of an     |
 |      ================   IEEE exception handler.             |
 |                         The final enabled status is         |
 |                         specified by 'mode'.                |
 |      code   : one of                                        |
 |                IEEE_DIV_BY_ZERO = DIVision BY ZERO          |
 |                IEEE_INEXACT     = INEXACT result            |
 |                IEEE_INV_OP      = INValid OPeration         |
 |                IEEE_OVERFLOW    = exponent OVERFLOW         |
 |                IEEE_UNDERFLOW   = exponent UNDERFLOW        |
 |      mode   : TRUE  = trap handler enabled                  |
 |               FALSE = trap handler disabled                 |
 ---------------------------------------------------------------}
GLOBAL PROCEDURE IEEE_trap_enable(code : INTEGER; mode : BOOLEAN);
BEGIN
   CASE code OF
    IEEE_INV_OP      : IF mode THEN e_sioe ELSE e_rioe;
    IEEE_DIV_BY_ZERO : IF mode THEN e_sdze ELSE e_rdze;
    IEEE_OVERFLOW    : IF mode THEN e_sofe ELSE e_rofe;
    IEEE_UNDERFLOW   : IF mode THeN e_sufe ELSE e_rufe;
    IEEE_INEXACT     : IF mode THEN e_siee ELSE e_riee;
   END;
END;

{---------------------------------------------------------------
 |      x_sqrt : square root function                          |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_sqrt(r : REAL) : REAL;         EXTERNAL r_sqrt;
GLOBAL FUNCTION x_sqrt(r : INTERVAL) : INTERVAL; EXTERNAL i_sqrt;

{---------------------------------------------------------------
 |      x_exp  : exponential function                          |
 |      =====    (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_exp(r : REAL) : REAL;         EXTERNAL r_exp;
GLOBAL FUNCTION x_exp(r : INTERVAL) : INTERVAL; EXTERNAL i_exp;

{---------------------------------------------------------------
 |      x_exp2 : exponential function with base 2              |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_exp2(r : REAL) : REAL;         EXTERNAL r_exp2;
GLOBAL FUNCTION x_exp2(r : INTERVAL) : INTERVAL; EXTERNAL i_exp2;

{---------------------------------------------------------------
 |      x_exp10 : exponential function with base 10            |
 |      =======   (using a posteriori error estimations)       |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_exp10(r : REAL) : REAL;         EXTERNAL r_ep10;
GLOBAL FUNCTION x_exp10(r : INTERVAL) : INTERVAL; EXTERNAL i_ep10;

{---------------------------------------------------------------
 |      x_ln   : natural logarithm function                    |
 |      ====     (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_ln(r : REAL) : REAL;         EXTERNAL r_log;
GLOBAL FUNCTION x_ln(r : INTERVAL) : INTERVAL; EXTERNAL i_log;

{---------------------------------------------------------------
 |      x_log2 : logarithm base 2 function                     |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_log2(r : REAL) : REAL;         EXTERNAL r_log2;
GLOBAL FUNCTION x_log2(r : INTERVAL) : INTERVAL; EXTERNAL i_log2;

{---------------------------------------------------------------
 |      x_log10 : logarithm base 10 function                   |
 |      =======   (using a posteriori error estimations)       |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_log10(r : REAL) : REAL;         EXTERNAL r_lg10;
GLOBAL FUNCTION x_log10(r : INTERVAL) : INTERVAL; EXTERNAL i_lg10;

{---------------------------------------------------------------
 |      x_sin : sine function                                  |
 |      =====   (using a posteriori error estimations)         |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_sin(r : REAL) : REAL;         EXTERNAL r_sin;
GLOBAL FUNCTION x_sin(r : INTERVAL) : INTERVAL; EXTERNAL i_sin;

{---------------------------------------------------------------
 |      x_cos : cosine function                                |
 |      =====   (using a posteriori error estimations)         |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_cos(r : REAL) : REAL;         EXTERNAL r_cos;
GLOBAL FUNCTION x_cos(r : INTERVAL) : INTERVAL; EXTERNAL i_cos;

{---------------------------------------------------------------
 |      x_tan : tangent function                               |
 |      =====   (using a posteriori error estimations)         |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_tan(r : REAL) : REAL;         EXTERNAL r_tan;
GLOBAL FUNCTION x_tan(r : INTERVAL) : INTERVAL; EXTERNAL i_tan;

{---------------------------------------------------------------
 |      x_cot : cotangent function                             |
 |      =====   (using a posteriori error estimations)         |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_cot(r : REAL) : REAL;         EXTERNAL r_cot;
GLOBAL FUNCTION x_cot(r : INTERVAL) : INTERVAL; EXTERNAL i_cot;

{---------------------------------------------------------------
 |      x_arcsin : arcus sine function                         |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arcsin(r : REAL) : REAL;         EXTERNAL r_asin;
GLOBAL FUNCTION x_arcsin(r : INTERVAL) : INTERVAL; EXTERNAL i_asin;

{---------------------------------------------------------------
 |      x_arccos : arcus cosine function                       |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arccos(r : REAL) : REAL;         EXTERNAL r_acos;
GLOBAL FUNCTION x_arccos(r : INTERVAL) : INTERVAL; EXTERNAL i_acos;

{---------------------------------------------------------------
 |      x_arctan : arcus tangent function                      |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arctan(r : REAL) : REAL;         EXTERNAL r_atan;
GLOBAL FUNCTION x_arctan(r : INTERVAL) : INTERVAL; EXTERNAL i_atan;

{---------------------------------------------------------------
 |      x_arctan2 : arcus tangent function for 2 arguments     |
 |      =========   (using a posteriori error estimations)     |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arctan2(r1,r2 : REAL) : REAL;         EXTERNAL r_atn2;
GLOBAL FUNCTION x_arctan2(r1,r2 : INTERVAL) : INTERVAL; EXTERNAL i_atn2;

{---------------------------------------------------------------
 |      x_arccot : arcus cotangent function                    |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arccot(r : REAL) : REAL;         EXTERNAL r_acot;
GLOBAL FUNCTION x_arccot(r : INTERVAL) : INTERVAL; EXTERNAL i_acot;

{---------------------------------------------------------------
 |      x_sinh : hyperbolic sine function                      |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_sinh(r : REAL) : REAL;         EXTERNAL r_sinh;
GLOBAL FUNCTION x_sinh(r : INTERVAL) : INTERVAL; EXTERNAL i_sinh;

{---------------------------------------------------------------
 |      x_cosh : hyperbolic cosine function                    |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_cosh(r : REAL) : REAL;         EXTERNAL r_cosh;
GLOBAL FUNCTION x_cosh(r : INTERVAL) : INTERVAL; EXTERNAL i_cosh;

{---------------------------------------------------------------
 |      x_tanh : hyperbolic tangent function                   |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_tanh(r : REAL) : REAL;         EXTERNAL r_tanh;
GLOBAL FUNCTION x_tanh(r : INTERVAL) : INTERVAL; EXTERNAL i_tanh;

{---------------------------------------------------------------
 |      x_coth : hyperbolic cotangent function                 |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_coth(r : REAL) : REAL;         EXTERNAL r_coth;
GLOBAL FUNCTION x_coth(r : INTERVAL) : INTERVAL; EXTERNAL i_coth;

{---------------------------------------------------------------
 |      x_arsinh : area sine function                          |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arsinh(r : REAL) : REAL;         EXTERNAL r_asnh;
GLOBAL FUNCTION x_arsinh(r : INTERVAL) : INTERVAL; EXTERNAL i_asnh;

{---------------------------------------------------------------
 |      x_arcosh : area cosine function                        |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arcosh(r : REAL) : REAL;         EXTERNAL r_acsh;
GLOBAL FUNCTION x_arcosh(r : INTERVAL) : INTERVAL; EXTERNAL i_acsh;

{---------------------------------------------------------------
 |      x_artanh : area tangent function                       |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_artanh(r : REAL) : REAL;         EXTERNAL r_atnh;
GLOBAL FUNCTION x_artanh(r : INTERVAL) : INTERVAL; EXTERNAL i_atnh;

{---------------------------------------------------------------
 |      x_arcoth : area cotangent function                     |
 |      ========   (using a posteriori error estimations)      |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_arcoth(r : REAL) : REAL;         EXTERNAL r_acth;
GLOBAL FUNCTION x_arcoth(r : INTERVAL) : INTERVAL; EXTERNAL i_acth;

{---------------------------------------------------------------
 |      x_power : a to the x                                   |
 |      =======   (using a posteriori error estimations)       |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_power(a,x : REAL) : REAL;         EXTERNAL r_pow;
GLOBAL FUNCTION x_power(a,x : INTERVAL) : INTERVAL; EXTERNAL i_pow;

{---------------------------------------------------------------
 |      x_loga : logarithm of x to base a                      |
 |      ======   (using a posteriori error estimations)        |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_loga(x,a : REAL) : REAL;         EXTERNAL r_loga;
GLOBAL FUNCTION x_loga(x : INTERVAL; a:REAL) : INTERVAL; EXTERNAL i_loga;

{---------------------------------------------------------------
 |      x_rval : assign real hex string to a real              |
 |      ======                                                 |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_rval(x : STRING) : REAL; EXTERNAL s_rhex;

{---------------------------------------------------------------
 |      x_image : assign real to a hex string using mode 'x'   |
 |      ======    'X' (like WRITE)                             |
 ---------------------------------------------------------------}
GLOBAL FUNCTION x_image(x: REAL; mode: CHAR) : STRING; EXTERNAL s_whex;

END.
